@@ -20,29 +20,9 @@
 
 #include "dialog_client.h"
 
-/********************************************
- *
- * Experimental dialog dragging modifications
- */
-
-/* different dragging restraints */
-#define DIALOG_DRAG_NONE         1
-#define DIALOG_DRAG_RESTRAIN     2
-#define DIALOG_DRAG_FREE         3
-
-/* below can be changed for different behaviours */
-
-/* Selected restraint */
-#define DIALOG_DRAG_MODE         DIALOG_DRAG_FREE
-
-
-/********************************************/
-
-
 static void dialog_client_check_for_state_hints(Client *c);
 static void dialog_client_drag(Client *c);
 static void _get_mouse_position(Wm *w, int *x, int *y);
-static void _draw_outline(Client *c, int x, int y, int w, int h);
 
 Client*
 dialog_client_new(Wm *w, Window win, Client *trans)
@@ -123,6 +103,8 @@ dialog_client_check_for_state_hints(Client *c)
 
       c->flags ^= CLIENT_IS_MODAL_FLAG;
 
+      c->wm->n_modals_present++;
+
       /* Call comp_engine_client_show to add damage to main window
        * so it gets fully lowlighted ok.
        */
@@ -162,8 +144,10 @@ dialog_client_move_resize(Client *c)
 
   base_client_move_resize(c);
 
-  XMoveResizeWindow(w->dpy, c->window, offset_west, frm_size,
-		    c->width, c->height);
+  if (c->window != c->frame) 	/* Non decorated */
+    XMoveResizeWindow(w->dpy, c->window, 
+		      offset_west, frm_size,
+		      c->width, c->height);
 
 #ifndef USE_COMPOSITE
    if (w->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
@@ -249,6 +233,9 @@ dialog_client_show(Client *c)
 		XMapSubwindows(w->dpy, p->frame);
 		XMapWindow(w->dpy, p->frame);
 	      }
+
+	  if (c->flags & CLIENT_IS_MODAL_FLAG)
+	    c->wm->n_modals_present++;
 	}
 
       if (c->win_modal_blocker)
@@ -319,7 +306,8 @@ dialog_client_show(Client *c)
 	    }
 	}
 
-      if (lowest_trans->win_group 
+      if (c->trans->type != MBCLIENT_TYPE_DIALOG
+	  && lowest_trans->win_group 
 	  && (lowest_trans->type == MBCLIENT_TYPE_DESKTOP
 	      || lowest_trans->type == MBCLIENT_TYPE_APP))
 	{
@@ -327,7 +315,7 @@ dialog_client_show(Client *c)
 
 	  visible = wm_get_visible_main_client(w);
 
-	  if (visible 
+	  if (visible
 	      && visible != lowest_trans 
 	      && visible->win_group == lowest_trans->win_group)
 	    stack_move_above_client(c, visible);
@@ -336,6 +324,7 @@ dialog_client_show(Client *c)
   else
     stack_move_top(c);
 
+
   /* Now move any transients for us above us */
 
   client_get_transient_list(w, &transient_list, c);
@@ -394,7 +383,7 @@ dialog_client_reparent(Client *c)
                 |CWBackPixel|CWBorderPixel|CWColormap ;
 #endif
 
-  dbg("%s() want lowlight : wm:%i , client:%i\n", __func__,
+  dbg("%s() want lowlight : wm:%i , client:%li\n", __func__,
       c->wm->config->dialog_shade, (c->flags & CLIENT_IS_MODAL_FLAG));
 #ifndef USE_COMPOSITE
   if (c->wm->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
@@ -682,18 +671,15 @@ dialog_init_geometry(Client *c)
     return;
 
   /* Allow decorationless dialogs to position themselves anywhere 
-   * But centered initially if 0,0 ( splash screens ).
-   *
-   * However for 'menu' dialogs this doesn't apply as a fullscreen
-   * window may want a menu at 0,0, so we dont center them
+   * But centered initially if 0,0 and a splash screen.
   */
-  if (c->flags & CLIENT_TITLE_HIDDEN_FLAG)
+  if (c->flags & (CLIENT_TITLE_HIDDEN_FLAG))
     {
-      if (!(c->flags & CLIENT_IS_MENU_DIALOG) && c->x == 0 && c->y == 0)
+      if ((c->flags & CLIENT_IS_SPLASH_WIN) && c->x == 0 && c->y == 0)
 	{
 	  if (c->height < w->dpy_height)
 	    c->y = (w->dpy_height - c->height)/2;
-
+      
 	  if (c->width < w->dpy_width)
 	    c->x = (w->dpy_width - c->width)/2;
 	}
@@ -781,7 +767,6 @@ dialog_init_geometry(Client *c)
    *   + positioned at 0,0
    *   + positioned offscreen
    */
-
   if ( (c->x - bdr_west) < avail_x 
        || (c->x + c->width + bdr_east) > (avail_x + avail_width))
     {
@@ -815,7 +800,6 @@ dialog_client_configure(Client *c)
   dbg("%s() client has menu hint: %s\n",
       __func__, (c->flags & CLIENT_IS_MENU_DIALOG) ? "yes" : "no");
 
-
   dialog_init_geometry(c);
 }
 
@@ -1012,11 +996,7 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 
    if (w->config->super_modal)
      {
-       Client *p;
-
-       p = client_get_highest_transient(c, CLIENT_IS_MODAL_FLAG, NULL);
-
-       if (p && p != c)
+       if (w->focused_client != c)
 	 {
 	   MBList *button_item = client_get_button_list_item_from_event(c, e);
 	   /* In the precense of a modal transient dialog ignore 
@@ -1068,53 +1048,21 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 	    if (e->window != c->frames_decor[NORTH])
 	      return;
 	    
-	    if (XGrabPointer(w->dpy, w->root, False,
-			     ButtonPressMask|ButtonReleaseMask,
-			     GrabModeAsync,
-			     GrabModeAsync, 
-			     None, w->curs, CurrentTime) == GrabSuccess)
+	    XUnmapWindow(w->dpy, c->frame);
+
+	    dialog_client_drag(c);
+
+	    misc_trap_xerrors(); 
+	    XMapWindow(w->dpy, c->frame);
+
+	    if (w->focused_client == c)
 	      {
-		XUnmapWindow(w->dpy, c->frame);
-		_draw_outline(c, c->x - offset_west, c->y - offset_north,
-			      c->width + offset_west + offset_east,
-			      c->height + offset_north + offset_south);
-		
-		XFlush(w->dpy);
-		
-		for (;;) 
-		  {
-		    XEvent xev;
-		    XMaskEvent(w->dpy,ButtonReleaseMask|ButtonPressMask, &xev);
-		    if (xev.type == ButtonRelease)
-		      {
-			_draw_outline(c, c->x - offset_west, 
-				      c->y - offset_north,
-				      c->width + offset_west + offset_east,
-				      c->height + offset_north + offset_south);
-			XMapWindow(w->dpy, c->frame);
-			break;
-		      }
-		  }
-		
-		XUngrabPointer(w->dpy, CurrentTime); 
-
-		/* For some odd reason the above make focus get 
-                 * lost ( for gtk apps at least ). We therefore
-                 * reset it.
-                 * ( we have to set focused client to NULL for set_focus()
-                 *   logic to work :( may a re_focus() is better ).
-		*/
-		if (w->focused_client == c)
-		  {
-		    w->focused_client = NULL;
-		    client_set_focus(c);
-		  }
-
-		XSync(w->dpy, False);
+		w->focused_client = NULL;
+		client_set_focus(c);
 	      }
+	    misc_untrap_xerrors(); 	    
 
-	    
-
+	    XSync(w->dpy, False);
 	    return;
 	  }
 
@@ -1126,11 +1074,15 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 static void
 dialog_client_drag(Client *c) /* drag box */
 {
-  XEvent ev;
-  int    x1, y1, old_cx = c->x, old_cy = c->y;
-  int    frm_size     = dialog_client_title_height(c);
-  int    offset_south = 0, offset_west = 0, offset_east = 0;
-  int    have_grab = 0;
+  Wm                  *w = c->wm;
+  XEvent               ev;
+  int                  offset_south = 0, offset_west = 0, offset_east = 0;
+  int                  x1, y1, old_cx = c->x, old_cy = c->y;
+  int                  frm_size     = dialog_client_title_height(c);
+  XSetWindowAttributes attr;
+  Window               win_outline;
+  XRectangle           rects[1];
+  Bool                 done = False, client_removed = False;
 
   dbg("%s called\n", __func__);
 
@@ -1143,118 +1095,105 @@ dialog_client_drag(Client *c) /* drag box */
       != GrabSuccess)
     return;
 
-  comp_engine_client_show(c->wm, c); 
+  attr.override_redirect = True;
+  attr.background_pixel  = BlackPixel(w->dpy, w->screen);  
 
-  c->flags |= CLIENT_IS_MOVING;
+  win_outline = XCreateWindow(w->dpy, 
+			      w->root,
+			      c->x - offset_west, c->y - frm_size,
+			      c->width + offset_west + offset_east,
+			      c->height + frm_size + offset_south,
+			      0,
+			      CopyFromParent, 
+			      CopyFromParent, 
+			      CopyFromParent,
+			      CWBackPixel|CWOverrideRedirect,
+			      &attr);
+
+  rects[0].x      = 2;  
+  rects[0].y      = 2;
+  rects[0].width  = c->width + offset_west + offset_east - 4;
+  rects[0].height = c->height + frm_size + offset_south  - 4;
+
+  XShapeCombineRectangles (w->dpy, win_outline,
+			   ShapeBounding,
+			   0, 0, rects, 1, ShapeSubtract, 0 );
+
+  XMapWindow (w->dpy, win_outline);
+  XSync (w->dpy, False);
+
+  comp_engine_client_show(c->wm, c); 
 
   _get_mouse_position(c->wm, &x1, &y1);
 
   XFlush(c->wm->dpy);
 
-  for (;;) 
+  while (!done) 
     {
       int wanted_x = 0, wanted_y = 0;
 
-      if (!have_grab) 
-	{
-	  _draw_outline(c, c->x - offset_west, c->y - frm_size,
-			c->width + offset_west + offset_east,
-			c->height + frm_size + offset_south);
-	  XGrabServer(c->wm->dpy); 
-	  have_grab = 1; 
-	}
-
-
       XMaskEvent(c->wm->dpy, 
-		 ButtonPressMask|ButtonReleaseMask|PointerMotionMask,
+		 ButtonPressMask|ButtonReleaseMask
+		 |PointerMotionMask|SubstructureNotifyMask,
 		 &ev);
 
     switch (ev.type) 
       {
+      case MapRequest:
+	wm_handle_map_request(w, &ev.xmaprequest); 
+	break;
+
+      case UnmapNotify:
+	{
+	  if (wm_find_client(w, ev.xunmap.window, WINDOW) == c)
+	    {
+	      client_removed = True;
+	      done = True;
+	    }
+
+	  wm_handle_unmap_event(w, &ev.xunmap); 
+	  break;
+	}
+
       case MotionNotify:
-	  
-	_draw_outline(c, 
-		      c->x - offset_west, 
-		      c->y - frm_size,
-		      c->width + offset_west + offset_east,
-		      c->height + frm_size + offset_south);
+	if (w->config->dialog_stratergy == WM_DIALOGS_STRATERGY_STATIC)
+	  break;
 
 	wanted_x = (old_cx + (ev.xmotion.x - x1));
 	wanted_y = (old_cy + (ev.xmotion.y - y1));
 
-	switch (DIALOG_DRAG_MODE) 
-	  {
-	  case DIALOG_DRAG_RESTRAIN:
+	c->x = wanted_x;
+	c->y = wanted_y;
 
-	    if ( (wanted_x - offset_west) < 0)
-	      c->x = offset_west;
-	    else if ( (wanted_x + c->width + offset_east) > c->wm->dpy_width)
-	      c->x = c->wm->dpy_width - (c->width + offset_east);
-	    else c->x = wanted_x;
-	    
-	    if ( (wanted_y - frm_size) < 0)
-	      c->y = frm_size;
-	    else if ( (wanted_y + c->height + offset_south) > c->wm->dpy_height)
-	      c->y = c->wm->dpy_height - (c->height + offset_south);
-	    else c->y = wanted_y;
-
-	    break;
-          case DIALOG_DRAG_FREE:
-	    c->x = wanted_x;
-	    c->y = wanted_y;
-	    break;
-	  default:
-	    break;
-	  }
-
-	_draw_outline(c, c->x - offset_west, c->y - frm_size,
-		      c->width + offset_west + offset_east,
-		      c->height + frm_size + offset_south);
+	XMoveWindow(c->wm->dpy, win_outline, c->x-offset_west, c->y-frm_size);
 	break;
-	
+
       case ButtonRelease:
-	dbg("drag, got release");
-	_draw_outline(c, c->x - offset_west, c->y - frm_size,
-		      c->width + offset_west + offset_east,
-		      c->height + frm_size + offset_south);
-
-#ifndef USE_COMPOSITE	
-
-#if 0
-	if (c->wm->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
-	  {
-	    XMoveResizeWindow(c->wm->dpy, 
-			      c->window, 
-			      c->x, 
-			      c->y, 
-			      c->width,
-			      c->height);
-	  } 
-	else
-#endif
-
-#endif /* USE_COMPOSITE */
-	  {
-	    XMoveWindow(c->wm->dpy, c->frame, c->x - offset_west,
-			c->y - dialog_client_title_height(c));
-	  }
-	
-	c->show(c);
-	
-	c->flags &= ~ CLIENT_IS_MOVING;
-	
-	XUngrabPointer(c->wm->dpy, CurrentTime);
-	XUngrabServer(c->wm->dpy);
-	wm_activate_client(c);
-	return;
+	XMoveWindow(c->wm->dpy, c->frame, c->x - offset_west,
+		    c->y - dialog_client_title_height(c));
+	done = True;
+	break;
       }
     }
 
-  XUngrabPointer(c->wm->dpy, CurrentTime);
-  XUngrabServer(c->wm->dpy);
 
+  XUngrabPointer(w->dpy, CurrentTime);
 
-  client_deliver_config(c);
+  misc_trap_xerrors(); 
+  XDestroyWindow (w->dpy, win_outline);
+
+  if (client_removed == False) 
+    {
+      c->show(c);
+      
+      if (w->config->dialog_stratergy != WM_DIALOGS_STRATERGY_STATIC)
+	{
+	  wm_activate_client(c);
+	  client_deliver_config(c);
+	}
+    }
+
+  misc_untrap_xerrors();
 }
 
 
@@ -1269,16 +1208,6 @@ _get_mouse_position(Wm *w, int *x, int *y)
 		x, y, &win_x, &win_y, &mask);
 }
 
-static void
-_draw_outline(Client *c, int x, int y, int width, int height)
-{
-  dbg("%s called +%i,+%i %ix%i\n", __func__,x,y,width,height);
-
-  XDrawRectangle(c->wm->dpy, c->wm->root, c->wm->mbtheme->band_gc, 
-		 x-1, y-1, width+2, height+2);
-
-}
-
 static Client*
 dialog_client_set_focus_next(Client *c)
 {
@@ -1301,10 +1230,37 @@ dialog_client_set_focus_next(Client *c)
     }
   else
     {
+      Client *app_client = NULL, *p = NULL;
+
       dbg("%s setting focus to visibe_main\n", __func__); 
+
       if (w->focused_client == c)
 	w->focused_client = NULL;
-      return wm_get_visible_main_client(w);
+
+      /* Find highest modal */
+      stack_enumerate_reverse (w,p)
+	{
+	  if (p->type == MBCLIENT_TYPE_DIALOG
+	      && (p->flags & CLIENT_IS_MODAL_FLAG)
+	      && p != c)
+	    return p;
+	}
+
+      /* Check if main window has modals and focus that */
+      app_client = wm_get_visible_main_client(w);
+
+      if (app_client)
+	{
+	  Client *modal = NULL;
+
+	  modal = client_get_highest_transient(app_client, 
+					       CLIENT_IS_MODAL_FLAG, c);
+
+	  if (modal)
+	    return modal;
+	}
+
+      return app_client;
     }
 
   return NULL;
@@ -1321,6 +1277,9 @@ dialog_client_iconize(Client *c)
   c->mapped = False;
   XUnmapWindow(w->dpy, c->frame); 
 
+  if (c->flags & CLIENT_IS_MODAL_FLAG)
+    c->wm->n_modals_present--;
+
   if (c->win_modal_blocker)
     {
       XUnmapWindow(w->dpy, c->win_modal_blocker);
@@ -1344,6 +1303,9 @@ dialog_client_destroy(Client *c)
   /* Focus the saved next or return a likely candidate if none found */
   d = dialog_client_set_focus_next(c);
 
+  if (c->flags & CLIENT_IS_MODAL_FLAG)      
+    c->wm->n_modals_present--;
+
   if (c->win_modal_blocker)
     {
       Wm *w = c->wm;
