@@ -1,17 +1,22 @@
-/* matchbox - a lightweight window manager
-
-   Copyright 2002 Matthew Allum
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-*/
+/* 
+ *  Matchbox Window Manager - A lightweight window manager not for the
+ *                            desktop.
+ *
+ *  Authored By Matthew Allum <mallum@o-hand.com>
+ *
+ *  Copyright (c) 2002, 2004 OpenedHand Ltd - http://o-hand.com
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
 
 #ifndef _MBTHEME_H_
 #define _MBTHEME_H_
@@ -95,9 +100,6 @@ typedef struct _mb_theme_layer {
   MBColor  *color;
   MBPixbufImage *img;
   MBThemeLabel  *label;  
-  MBThemeLabel  *sublabel;  
-
-
   
   MBColor  *color_end; 	/* for gradients */
   
@@ -156,9 +158,7 @@ typedef struct _mbtheme {
   struct list_item* colors;
   struct list_item* fonts;
 
-  GC gc;
-  GC mask_gc;
-  GC band_gc; 	/* for drag window  */
+  GC                gc, mask_gc, band_gc; /* for drag window  */
 
   MBPixbufImage* img_caches[N_FRAME_TYPES];
 
@@ -169,15 +169,18 @@ typedef struct _mbtheme {
   MBThemeParam *toolbar_panel_w;
   MBThemeParam *toolbar_panel_h;
 
-#ifdef USE_EXTRAS
-
   /* For image substitution */
   char           subst_char;
   MBPixbufImage *subst_img;
 
-#endif
+  /* App side decoration pixmap cache */
 
-  struct _wm *wm;
+  Pixmap app_win_pxm_cache[3];
+
+  /* disable cacheing, not recommened */
+  Bool           disable_pixbuf_cache;
+
+  struct _wm    *wm;
    
 } MBTheme;
 
@@ -218,6 +221,12 @@ int
 theme_frame_defined_height_get (MBTheme       *theme,
 				int            frame_ref );
 
+Bool
+theme_has_message_decor( MBTheme *theme );
+
+Bool
+theme_has_borders_only_decor( MBTheme *theme );
+
 void     
 theme_img_cache_clear (MBTheme       *theme,
 		       int            frame_ref );
@@ -225,6 +234,10 @@ theme_img_cache_clear (MBTheme       *theme,
 void     
 theme_img_cache_clear_all (MBTheme       *theme );
 
+void
+theme_pixmap_cache_clear_all( MBTheme *theme );
+
+
 void     
 theme_frame_button_paint (MBTheme       *theme,
 			  Client        *c,
@@ -238,8 +251,6 @@ Bool
 theme_frame_paint (MBTheme       *theme,
 		   Client        *c,
 		   int            frame_ref,
-		   int            dx,
-		   int            dy,
 		   int            dw,
 		   int            dh );
 
