@@ -21,6 +21,14 @@
 #include "wm.h"
 #include "config.h"
 
+#ifdef HAVE_XFIXES
+#include <X11/extensions/Xfixes.h> /* Used to *really* hide cursor */
+#endif
+
+#ifdef HAVE_XCURSOR
+#include <X11/Xcursor/Xcursor.h>
+#endif
+
 #ifdef USE_XSETTINGS
 static void wm_xsettings_notify_cb (const char       *name,
 				    XSettingsAction   action,
@@ -1005,6 +1013,17 @@ wm_handle_button_event(Wm *w, XButtonEvent *e)
      {
        /* Click was on window rather than decorations */
 
+       /* Dont move focus for a modal dialog */
+       if (w->focused_client
+           && c != w->focused_client
+           && w->focused_client->type == MBCLIENT_TYPE_DIALOG
+           && w->focused_client->trans == c
+           && w->focused_client->flags & CLIENT_IS_MODAL_FLAG)
+         {
+           XAllowEvents(w->dpy, ReplayPointer, CurrentTime);
+           return;
+         }
+
        if (c->type == MBCLIENT_TYPE_DIALOG 
 	   && w->config->dialog_stratergy != WM_DIALOGS_STRATERGY_STATIC)
 	 {
@@ -1351,8 +1370,12 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
    if (!(value_mask & CWWidth))  req_w = c->width;
    if (!(value_mask & CWHeight)) req_h = c->height;
    
-   if (e->x <= 0 || !(value_mask & CWX)) req_x = c->x;
-   if (e->y <= 0 || !(value_mask & CWY)) req_y = c->y;
+   if ((e->x <= 0 && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
+	|| !(value_mask & CWX)) 
+       req_x = c->x;
+   if ((e->y <= 0 && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
+       || !(value_mask & CWY)) 
+     req_y = c->y;
 
    dbg("%s() for win %s - have w: %i vs %i, h: %i" 
        "vs %i, x: %i vs %i, y: %i vs %i,\n", 
@@ -1462,6 +1485,7 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
         * - eg toolbar/panel/input windows may dissapear and make
         *      more space available. 
         */
+
        if (e->width && (value_mask & CWWidth) 
 	   && e->width != c->width && e->width != c->init_width)
 	 c->init_width = e->width;
@@ -1512,6 +1536,32 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 
 	   dialog_constrain_geometry(c, &req_x, &req_y, &req_w, &req_h);
 
+#ifdef USE_ALT_INPUT_WIN
+	 /* Alternate Input windows use the dialog type to 
+	  * fit in with stacking etc but they cant change width, x nor y
+	  */
+	 if (c->flags & (CLIENT_TB_ALT_TRANS_FOR_DIALOG
+	                   |CLIENT_TB_ALT_TRANS_FOR_APP))
+	   {
+	     if (c->height != req_h)
+	       {
+		 Client *trans_client = c->trans;
+		 int     diff = c->height - req_h;
+
+		 c->height  = req_h;
+		 c->y      += diff;
+		 toolbar_client_move_resize(c);
+
+		 if (trans_client)
+		   {
+		     trans_client->height += diff;
+		     trans_client->move_resize(trans_client);
+		     trans_client->redraw(trans_client, False);
+		   }
+	       }
+	     return;
+	   }
+#endif
 	   if (c->width == req_w && c->height == req_h)
 	     want_fake_configure = True; 
 
@@ -1519,16 +1569,11 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	   if (c->width != req_w)
 	     client_buttons_delete_all(c);
 
-	   /* TODO: fix composite for this */
-	   // comp_engine_client_hide(c->wm, c);
-	   
 	   xwc.width  = c->width  = req_w;
 	   xwc.height = c->height = req_h;
 	   xwc.x      = c->x      = req_x;
 	   xwc.y      = c->y      = req_y; 
 
-	   dbg("GRAVITY: x:%i y:%i\n", c->x, c->y);
-
 	   /* ICCCM says if window only moved - not size change,
             * then we send a fake one too.
 	    */
@@ -1837,7 +1882,7 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 
       client_get_wm_protocols(c);
 
-      dbg("%s() WM_PROTOCOLS changed, flags now %i\n", __func__, c->flags);
+      dbg("%s() WM_PROTOCOLS changed, flags now %li\n", __func__, c->flags);
 
       /* If flags have changed, likely means a WM_CONTEXT_HELP|etc has changed 
        * and thus titlebar needs a repaint.
@@ -1954,23 +1999,26 @@ wm_make_new_client(Wm *w, Window win)
 		   dbg("%s() got splash atom\n", __func__ );
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
-		   c->flags ^= CLIENT_TITLE_HIDDEN_FLAG;
+		   c->flags |= (CLIENT_TITLE_HIDDEN_FLAG|CLIENT_IS_SPLASH_WIN);
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG]
-			|| value[0] == w->atoms[WINDOW_TYPE_MENU])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
 		 {
 		   dbg("%s() got type dialog atom\n", __func__ );
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
-
-		   /* check to see if we're a 'menu' dialog  */
-		   if (value[0] == w->atoms[WINDOW_TYPE_MENU])
-		     {
-		       dbg("%s() Got menu hint\n", __func__);
-		       c->flags |= CLIENT_IS_MENU_DIALOG;
-		     }
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_MENU]
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_POPUP_MENU]
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_DROPDOWN_MENU])
+		 {
+		   dbg("%s() got type menu atom\n", __func__ );
+		   c = dialog_client_new(w, win, NULL);
+		   if (c == NULL) goto end;
+		   c->flags |= CLIENT_IS_MENU_DIALOG;
+		 }
+	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE] 
+			/* Above to depricate for below */
+			|| value[0] == w->atoms[_NET_WM_WINDOW_TYPE_NOTIFICATION])
 		 {
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
@@ -1997,7 +2045,7 @@ wm_make_new_client(Wm *w, Window win)
        if (c) 
 	 c->flags ^= mwm_flags;
 
-       dbg("%s() got MWM flags: %i\n", __func__, c->flags );
+       dbg("%s() got MWM flags: %li\n", __func__, c->flags );
      }
 
    /* check for transient - ie detect if its a dialog */
@@ -2152,6 +2200,13 @@ wm_remove_client(Wm *w, Client *c)
 
   c->destroy(c);
 
+  /* If no clients left, force update of active props accordingly */
+  if (stack_empty(w) || w->stack_top_app == NULL)
+    {
+      ewmh_set_current_app_window(w);
+      ewmh_set_active(w);
+    }
+
   XUngrabServer(w->dpy);
 }
 
@@ -2183,6 +2238,8 @@ wm_update_layout(Wm         *w,
 	     switch (p->type)
 	       {
 	       case MBCLIENT_TYPE_APP :
+		 if (p->flags & CLIENT_FULLSCREEN_FLAG)
+		   break;
 		 p->width += change_amount;
 		 p->x     -= change_amount;
 		 p->move_resize(p);
@@ -2233,6 +2290,8 @@ wm_update_layout(Wm         *w,
 	     switch (p->type)
 	       {
 	       case MBCLIENT_TYPE_APP :
+		 if (p->flags & CLIENT_FULLSCREEN_FLAG)
+		   break;
 		 p->width += change_amount;
 		 p->move_resize(p);
 		 client_deliver_config(p);
@@ -2286,6 +2345,8 @@ wm_update_layout(Wm         *w,
 	     switch (p->type)
 	       {
 	       case MBCLIENT_TYPE_APP :
+		 if (p->flags & CLIENT_FULLSCREEN_FLAG)
+		   break;
 		 p->height += change_amount;
 		 p->y      -= change_amount;
 		 p->move_resize(p);
@@ -2332,6 +2393,9 @@ wm_update_layout(Wm         *w,
 	     switch (p->type)
 	       {
 	       case MBCLIENT_TYPE_APP :
+		 if (p->flags & CLIENT_FULLSCREEN_FLAG
+		     && client_changed->type == MBCLIENT_TYPE_PANEL)
+		   break;
 		 p->height += change_amount;
 		 p->move_resize(p);
 		 /*
@@ -2371,6 +2435,7 @@ wm_update_layout(Wm         *w,
    {
      if (p->type == MBCLIENT_TYPE_DIALOG) 
        {
+	 Bool force_height = False;
 	 int req_x = p->x, req_y = p->y, req_w = p->width, req_h = p->height;
 
 #ifdef USE_ALT_INPUT_WIN
@@ -2386,8 +2451,20 @@ wm_update_layout(Wm         *w,
 	     client_deliver_config(p);
 	     continue;
 	   }
+#else
+	 /* Always try and back to intially requested height, a  
+          * panel or input win may have dissapeared giving more 
+          * space.
+          * NOTE: we avoid doing this for ALT_INPUT_WINS (maemo)
+	 */
+	 if (p->init_height && p->init_height > req_h)
+	   {
+	     req_h = p->init_height;
+	     force_height = True;
+	   }
 #endif
-	 if (!dialog_constrain_geometry(p, &req_x, &req_y, &req_w, &req_h))
+	 if (!dialog_constrain_geometry(p, &req_x, &req_y, &req_w, &req_h)
+	     || force_height)
 	   {
 	     p->x = req_x; p->y = req_y; p->width = req_w; p->height = req_h;
 	     p->move_resize(p);
@@ -2475,7 +2552,6 @@ wm_activate_client(Client *c)
       /* As matchbox works around 'main' windows ( apps/main and desktop wins).
 	 We need to sync extra stuff up when displaying a new one.
        */
-
       Bool switching_from_to_fullscreen = False;
 
       /* save focus state for transient dialogs of prev showing main win */
@@ -2502,11 +2578,21 @@ wm_activate_client(Client *c)
          that way.                                                      */
 
       if (w->focused_client 
-	  && w->focused_client->type == MBCLIENT_TYPE_DIALOG
-	  && w->focused_client->trans == NULL)
+	  && w->focused_client->type == MBCLIENT_TYPE_DIALOG)
 	{
-	  dbg("%s() keeping focus to root dialog\n", __func__);
-	  client_to_focus = w->focused_client;
+	  Client *lowest_dialog = w->focused_client;
+
+	  /* Get lowest transient dialog, and check its trans for root  */
+	  while (lowest_dialog->trans != NULL 
+		 && lowest_dialog->type == MBCLIENT_TYPE_DIALOG)
+	    lowest_dialog = lowest_dialog->trans;
+
+	  if (lowest_dialog->trans == NULL 
+	      && lowest_dialog->type == MBCLIENT_TYPE_DIALOG)
+	    {
+	      dbg("%s() keeping focus to root dialog\n", __func__);
+	      client_to_focus = w->focused_client;
+	    }
 	}
 
       /* Raise panel + toolbars just above app but below app dialogs */
@@ -2607,16 +2693,19 @@ wm_activate_client(Client *c)
        * not currently at the top of the stack. In which case
        * dont try and focus it.
       */
-      if (c->trans != NULL) 	/* not trans for root */
+      if (c->trans != NULL && wm_get_visible_main_client(w))   
 	{
 	  Client *dialog_parent = c->trans;
 
+	  top_main_client = wm_get_visible_main_client(w);
+
 	  while (dialog_parent->trans != NULL)
 	    dialog_parent = dialog_parent->trans;
 
 	  if ( (dialog_parent->type == MBCLIENT_TYPE_APP
 		|| dialog_parent->type == MBCLIENT_TYPE_DESKTOP)
-	       && wm_get_visible_main_client(w) != dialog_parent)
+	       && top_main_client != dialog_parent
+	       && top_main_client->win_group != c->win_group)
 	    client_to_focus = NULL;
 	}
     }
@@ -2629,7 +2718,6 @@ wm_activate_client(Client *c)
       && mbtheme_has_titlebar_panel(w->mbtheme)
       && !(w->have_titlebar_panel->flags & CLIENT_DOCK_TITLEBAR_SHOW_ON_DESKTOP))
     {
-      // stack_move_below_client(c, w->client_desktop);
       /* TODO: Change to stack move bottom ? 
        *
        * eg use stack_move_above_client(c, NULL)
@@ -2642,8 +2730,6 @@ wm_activate_client(Client *c)
   if (set_desktop_show_hint) 
     ewmh_update_desktop_hint(w);
 
-  stack_dump(w);
-
   dbg("%s() now syncing above window stack\n", __func__);
 
   stack_sync_to_display(w);
@@ -2657,6 +2743,8 @@ wm_activate_client(Client *c)
 
   client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
 
+  stack_dump(w);
+
   if (set_current_app_active_hint)
     ewmh_set_current_app_window(w);
 
@@ -2800,23 +2888,57 @@ wm_toggle_desktop(Wm *w)
 void
 wm_set_cursor_visibility(Wm *w, Bool visible)
 {
-  /* TODO: do we need to free the cursors ? */
-  if (visible)
+#ifdef HAVE_XFIXES
+  int major = 0, minor = 0, ev_base, err_base;
+
+  if (XFixesQueryExtension (w->dpy, &ev_base, &err_base))
+    XFixesQueryVersion (w->dpy, &major, &minor);
+
+  dbg("Checking Xfixes, got %i.%i , ev (%i), err (%i)",
+      major, minor, ev_base, err_base);
+
+  if (major >= 4)
     {
-      w->config->no_cursor = False;
       w->curs = XCreateFontCursor(w->dpy, XC_left_ptr);
+      XDefineCursor(w->dpy, w->root, w->curs);
+
+      if (visible)
+	{
+	  w->config->no_cursor = False;
+	  misc_trap_xerrors();
+	  /* Appears to fire X error if not already hidden 
+           * but no way to query that ?
+	  */
+	  XFixesShowCursor (w->dpy, w->root);
+	  XSync(w->dpy, False);
+	  misc_untrap_xerrors();
+	}
+      else
+	{
+	  XFixesHideCursor (w->dpy, w->root);
+	}
     }
   else
+#endif
     {
-      Pixmap pix = XCreatePixmap (w->dpy, w->root, 1, 1, 1);
-      XColor col;
-      memset (&col, 0, sizeof (col));
-      w->blank_curs = XCreatePixmapCursor (w->dpy, pix, pix, &col, &col, 1, 1);
-      w->curs = w->blank_curs;
-      XFreePixmap (w->dpy, pix);
-      w->config->no_cursor = True;
-    }     
-   XDefineCursor(w->dpy, w->root, w->curs);
+      /* FIXME: do we need to free the cursors ? */
+      if (visible)
+	{
+	  w->config->no_cursor = False;
+	  w->curs = XCreateFontCursor(w->dpy, XC_left_ptr);
+	}
+      else
+	{
+	  Pixmap pix = XCreatePixmap (w->dpy, w->root, 1, 1, 1);
+	  XColor col;
+	  memset (&col, 0, sizeof (col));
+	  w->blank_curs = XCreatePixmapCursor (w->dpy, pix, pix, &col, &col, 1, 1);
+	  w->curs = w->blank_curs;
+	  XFreePixmap (w->dpy, pix);
+	  w->config->no_cursor = True;
+	}     
+      XDefineCursor(w->dpy, w->root, w->curs);
+    }
 }
 
 Client *
@@ -2834,6 +2956,8 @@ wm_get_desktop(Wm *w)
 #define XSET_LOWLIGHT   3
 #define XSET_TITLEBARS  4
 #define XSET_COMPOSITE  5
+#define XSET_CURSOR_THEME_NAME 6
+#define XSET_CURSOR_THEME_SIZE 7
 
 static void
 wm_xsettings_notify_cb (const char       *name,
@@ -2851,14 +2975,15 @@ wm_xsettings_notify_cb (const char       *name,
     { "MATCHBOX/CURSOR",    XSET_CURSOR    },
     { "MATCHBOX/TITLEBARS", XSET_TITLEBARS }, /* XXX Not implemeted */
     { "MATCHBOX/COMPOSITE", XSET_COMPOSITE },
+    { "Gtk/CursorThemeName", XSET_CURSOR_THEME_NAME },
+    { "Gtk/CursorThemeSize", XSET_CURSOR_THEME_SIZE },
     { NULL,       -1 } 
   };
 
   while(  mb_xsettings[i].name != NULL )
     {
       if (!strcmp(name, mb_xsettings[i].name)
-	  && setting != NULL 	/* XXX set to NULL when action deleted */
-	  && setting->type == XSETTINGS_TYPE_STRING )
+	  && setting != NULL) 	/* Set to NULL when action deleted */
 	{
 	  key = mb_xsettings[i].value;
 	  break;
@@ -2868,44 +2993,66 @@ wm_xsettings_notify_cb (const char       *name,
     
   if (key == XSET_UNKNOWN) return;
 
-  switch (action)
+  if (setting->type == XSETTINGS_TYPE_STRING)
     {
-    case XSETTINGS_ACTION_NEW:
-    case XSETTINGS_ACTION_CHANGED:
-      switch (key)
+      switch (action)
 	{
-	case XSET_COMPOSITE:
-	  if (!strcasecmp("off", setting->data.v_string)
-	      || !strcasecmp("false", setting->data.v_string))
+	case XSETTINGS_ACTION_NEW:
+	case XSETTINGS_ACTION_CHANGED:
+	  switch (key)
 	    {
-	      comp_engine_deinit(w);
-	    }
-	  else
-	    { 
-	      comp_engine_reinit(w);
+	    case XSET_COMPOSITE:
+	      if (!strcasecmp("off", setting->data.v_string)
+		  || !strcasecmp("false", setting->data.v_string))
+		{
+		  comp_engine_deinit(w);
+		}
+	      else
+		{ 
+		  comp_engine_reinit(w);
+		}
+	      break;
+	    case XSET_THEME:
+	      if (w->flags & STARTUP_FLAG)
+		w->config->theme = strdup(setting->data.v_string);
+	      else
+		mbtheme_switch(w, setting->data.v_string);
+	      break;
+	    case XSET_CURSOR:
+	      if (!strcasecmp("true", setting->data.v_string))
+		wm_set_cursor_visibility(w, True);
+	      else 
+		wm_set_cursor_visibility(w, False);
+	      break;
+	    case XSET_TITLEBARS:
+	      /* XXX todo */
+	      break;
+	    case XSET_CURSOR_THEME_NAME:
+#ifdef HAVE_XCURSOR
+	      XcursorSetTheme (w->dpy, setting->data.v_string);
+#endif
+	      break;
+	    default:
+	      break;
 	    }
+	case XSETTINGS_ACTION_DELETED:
+	  /* Do nothing for now */
 	  break;
-	case XSET_THEME:
-	  if (w->flags & STARTUP_FLAG)
-	      w->config->theme = strdup(setting->data.v_string);
-	  else
-	      mbtheme_switch(w, setting->data.v_string);
-	  break;
-	case XSET_CURSOR:
-	  if (!strcasecmp("true", setting->data.v_string))
-	    wm_set_cursor_visibility(w, True);
-	  else 
-	    wm_set_cursor_visibility(w, False);
-	  break;
-	case XSET_TITLEBARS:
-	  /* XXX todo */
-	  break;
-
 	}
-    case XSETTINGS_ACTION_DELETED:
-      /* Do nothing for now */
-      break;
     }
+  else if (setting->type == XSETTINGS_TYPE_INT)
+    {
+#ifdef HAVE_XCURSOR
+      if ( (action == XSETTINGS_ACTION_NEW 
+	    || action == XSETTINGS_ACTION_CHANGED) 
+	   && key == XSET_CURSOR_THEME_NAME)
+	{
+	  if (setting->data.v_int)
+	    XcursorSetDefaultSize (w->dpy, setting->data.v_int);
+	}
+#endif
+    }
+
 }
 
 #endif
